@@ -1,7 +1,7 @@
 /// Reads or writes a value owned by a source-of-truth.
 pub trait Binding<S>: Clone + 'static {
-    fn with<T, F: Fn(&S) -> T>(&self, f: F) -> T;
-    fn with_mut<T, F: Fn(&mut S) -> T>(&self, f: F) -> T;
+    fn with<T, F: FnOnce(&S) -> T>(&self, f: F) -> T;
+    fn with_mut<T, F: FnOnce(&mut S) -> T>(&self, f: F) -> T;
 
     fn get(&self) -> S
     where
@@ -11,10 +11,8 @@ pub trait Binding<S>: Clone + 'static {
     }
 
     fn set(&self, value: S)
-    where
-        S: Clone,
     {
-        self.with_mut(move |s| *s = value.clone());
+        self.with_mut(move |s| *s = value);
     }
 }
 
@@ -30,11 +28,11 @@ where
     Set: Fn(S) + Clone + 'static,
     S: Clone,
 {
-    fn with<T, F: Fn(&S) -> T>(&self, f: F) -> T {
+    fn with<T, F: FnOnce(&S) -> T>(&self, f: F) -> T {
         let v = (self.getf)();
         f(&v)
     }
-    fn with_mut<T, F: Fn(&mut S) -> T>(&self, f: F) -> T {
+    fn with_mut<T, F: FnOnce(&mut S) -> T>(&self, f: F) -> T {
         let mut v = (self.getf)();
         let t = f(&mut v);
         (self.setf)(v);
@@ -59,7 +57,7 @@ macro_rules! bind {
         let state1 = sref.clone();
         let state2 = sref.clone();
         Map {
-            getf: move || state1.get().$field.clone(),
+            getf: move || state1.with(|v| v.$field.clone()),
             setf: move |val| {
                 state2.with_mut(|v| v.$field = val);
             },
@@ -71,7 +69,7 @@ macro_rules! bind {
         let state2 = sref.clone();
         let idx = $index;
         Map {
-            getf: move || state1.get().$field[idx].clone(),
+            getf: move || state1.with(|v| v.$field[idx].clone()),
             setf: move |val| {
                 state2.with_mut(|v| v.$field[idx] = val);
             },
@@ -83,11 +81,9 @@ macro_rules! bind {
         let state2 = sref.clone();
         let idx = $index;
         Map {
-            getf: move || state1.get()[idx].clone(),
+            getf: move || state1.with(|v| v[idx].clone()),
             setf: move |val| {
-                let mut s = state2.get();
-                s[idx] = val;
-                state2.set(s);
+                state2.with_mut(|v| v[idx] = val);
             },
         }
     }};
@@ -118,10 +114,10 @@ macro_rules! bind_no_clone {
         where
             B: Binding<$type>,
         {
-            fn with<T, F: Fn(&$type2) -> T>(&self, f: F) -> T {
+            fn with<T, F: FnOnce(&$type2) -> T>(&self, f: F) -> T {
                 self.binding.with(|v| f(&v.$field))
             }
-            fn with_mut<T, F: Fn(&mut $type2) -> T>(&self, f: F) -> T {
+            fn with_mut<T, F: FnOnce(&mut $type2) -> T>(&self, f: F) -> T {
                 self.binding.with_mut(|v| f(&mut v.$field))
             }
         }
@@ -130,7 +126,7 @@ macro_rules! bind_no_clone {
     }};
 }
 
-// WIP. Attempting again to creating nice bindings without mutation.
+// WIP. Attempting again to creating nice bindings without cloning.
 #[derive(Clone)]
 struct Bnd2<B, L, T1> {
     binding: B,
@@ -149,11 +145,47 @@ where
     L: Lens<T1, T0> + Clone + 'static,
     T1: Clone + 'static
 {
-    fn with<T, F: Fn(&T0) -> T>(&self, f: F) -> T {
+    fn with<T, F: FnOnce(&T0) -> T>(&self, f: F) -> T {
         self.binding.with(|v| self.lens.with(v, |vv| f(vv)))
     }
-    fn with_mut<T, F: Fn(&mut T0) -> T>(&self, f: F) -> T {
+    fn with_mut<T, F: FnOnce(&mut T0) -> T>(&self, f: F) -> T {
         self.binding.with_mut(|v| self.lens.with_mut(v, |vv| f(vv)))
     }
 }
 
+
+#[derive(Clone)]
+pub struct Bnd3<L, Lmut> {
+    pub lens: L,
+    pub lens_mut: Lmut,
+}
+
+impl<S, L, Lmut> Binding<S> for Bnd3<L, Lmut>
+where
+    L: Fn(&dyn FnOnce(&S)) + Clone + 'static,
+    Lmut: Fn(&dyn FnOnce(&mut S)) + Clone + 'static
+{
+    fn with<T, F: FnOnce(&S) -> T>(&self, f: F) -> T {
+        let mut t = None;
+        (self.lens)(&|v| t = Some(f(v)));
+        t.unwrap()
+    }
+    fn with_mut<T, F: FnOnce(&mut S) -> T>(&self, f: F) -> T {
+        let mut t = None;
+        (self.lens_mut)(&|v| t = Some(f(v)));
+        t.unwrap()
+    }
+}
+
+#[macro_export]
+macro_rules! bind2 {
+    ( $state:expr, $field:ident ) => {{
+        let sref = &$state;
+        let state1 = sref.clone();
+        let state2 = sref.clone();
+        Bnd3 {
+            lens: move |f| state1.with(|x| f(x.$field)),
+            lens_mut: move |f| state2.with_mut(|x| f(x.$field)),
+        }
+    }};
+}
\ No newline at end of file
